function [alpha, c_l, c_d, c_m, idxOpts] = parsepolarinputs(optList, varargin)
    % PARSEPOLARINPUTS Parses the input and checks their validity.
    %   Returns the arrays for the main polar arrays (alpha, cl, cd, cm), as well as the index where
    %   the remaining options start.
    %   If some inputs were not provided (alpha, cl, cd, cm), the functions returns an empty array
    %   for those.
    %
    %   This function also contains checks on every "polar" input (i.e. the Polar strcture or the
    %   four arrays: alpha, cl, cd and cm) to ensure they are correct.
    %
    % -----
    % Usage:
    %   [alpha, cl, cd, cm, idxOpts] = PARSEPOLARINPUTS (optList)
    %   [alpha, cl, cd, cm, idxOpts] = PARSEPOLARINPUTS (optList, POLAR)
    %   [...] = PARSEPOLARINPUTS (optList, ALPHA)
    %   [...] = PARSEPOLARINPUTS (optList, ALPHA, CL, ...)
    %   [...] = PARSEPOLARINPUTS (optList, ALPHA, CL, CD, ...)
    %   [...] = PARSEPOLARINPUTS (optList, ALPHA, CL, CD, CM, ...)
    %
    % -----
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % Apache 2.0 License
    % https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox

    % ----------------------------------------------------------------------------------------------

    import af_tools.utils.*

    % First ensures the option list is properly set
    validateattributes(optList, {'char', 'string', 'cell'}, {}, mfilename(), 'optList', 1);

    % Parse inputs
    hasNoInput = isempty(varargin) || isoption(optList, varargin{1});
    if hasNoInput
        idxOpts = 1;
        % Prompt user for polar
        [filename, path] = uigetfile('*.mat', 'Select a Polar file');
        tmp = load([path, filename]);
        [alpha, c_l, c_d, c_m] = extractpolar(tmp.Polar);
        clear tmp;
    else
        if isstruct(varargin{1})
            idxOpts = 2;
            % Extract polar values
            Polar = varargin{1};
            validateattributes(Polar, {'struct'}, {'nonempty'}, mfilename(), 'Polar', 1);
            [alpha, c_l, c_d, c_m] = extractpolar(varargin{1});
        else
            [alpha, c_l, c_d, c_m, idxOpts] = parsearrays(optList, varargin);
        end
    end

    % Transform row vectors into column vectors
    alpha = vecttocol(alpha);
    c_l = vecttocol(c_l);
    c_d = vecttocol(c_d);
    c_m = vecttocol(c_m);

    % Validate input arrays or inputs coming from Polar structure
    validateattributes(alpha, {'numeric'}, {'real', '2d', 'nonempty', 'increasing'}, ...
                       mfilename(), 'alpha', 1);

    validatearray(c_l, size(alpha, 1), 2);
    validatearray(c_d, size(alpha, 1), 3);
    validatearray(c_m, size(alpha, 1), 4);

    % Standardize alpha size so we have one column per Polar as well
    if size(alpha, 2) == 1 && ~isempty(c_l)
        alpha = repmat(alpha, [1, size(c_l, 2)]);
    end

end

% --------------------------------------------------------------------------------------------------
function bool = isoption(optList, var)
    % ISOPTION Returns true if var is contained in option list.

    bool = any(strcmpi(var, optList));
end

% --------------------------------------------------------------------------------------------------
function [alpha, c_l, c_d, c_m] = extractpolar(Polar)
    % EXTRACTPOLAR Extract alpha, cl, cd and cm fields Polar structure.

    alpha = Polar.alpha;
    c_l = Polar.cl;
    c_d = Polar.cd;
    c_m = Polar.cm;
end

% --------------------------------------------------------------------------------------------------
function [var, idxOpts, foundOpts] = assignvar(optList, data, idx)
    % ASSIGNVAR Assign varargin value to a variable, ouptut corresponding idxOpts.

    var = [];

    foundOpts = false;
    if ~isoption(optList, data{idx})
        idxOpts = idx + 1;
        var = data{idx};
    else
        idxOpts = idx;
        foundOpts = true;
    end

end

% --------------------------------------------------------------------------------------------------
function [alpha, c_l, c_d, c_m, idxOpts] = parsearrays(optList, data)
    % PARSEARRAYS Parse user-provided arrays.

    alpha = [];
    c_l = [];
    c_d = [];
    c_m = [];

    if length(data) >= 1
        [alpha, idxOpts, foundOpts] = assignvar(optList, data, 1);
        if length(data) >= 2 && ~foundOpts
            [c_l, idxOpts, foundOpts] = assignvar(optList, data, 2);
            if length(data) >= 3 && ~foundOpts
                [c_d, idxOpts, foundOpts] = assignvar(optList, data, 3);
                if length(data) >= 4 && ~foundOpts
                    [c_m, idxOpts, ~] = assignvar(optList, data, 4);
                end
            end
        end
    end

end

% --------------------------------------------------------------------------------------------------
function validatearray(array, nrows, idx)
    % VALIDATEARRAY Checks type and diemnsions of array.

    if ~isempty(array)
        validateattributes(array, {'numeric'}, {'real', 'nrows', nrows, 'nonempty'}, ...
                           mfilename(), inputname(1), idx);
    end

end
