function [alpha_ext, c_l_ext, c_d_ext] = extendpolar(varargin)
    % EXTENDPOLAR Extends polars to the full range of AOA ([-180, 180] deg).
    %   This function implements different empirical methods to extend the polars outside of their
    %   usual range.
    %
    %   EXTENDPOLAR accepts inputs in the form of a Polar structure (generated with XF2MAT) or
    %   values for angles of attack and their associated cl and cd.
    %
    %   If the input is specified as aoa, cl and cd, the three arrays must have the same number of
    %   rows (one for each angle of attack). The columns indicate values for different polars
    %   (different Mach, different Reynolds,...).
    %   For example, if the cl values are given for 2 different Reynols and there are 100 values for
    %   the aoa, cl and cd must be [100 x 2].
    %
    % IMPORTANT:
    %   To be extended, the input polar must AT LEAST containt the points where AOA = 0 deg and
    %   stall point.
    %
    % <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (README)</a>
    % <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues">Report an issue</a>
    % -----
    %
    % Usage:
    %   [ALPHAEXT, CLEXT, CDEXT] = EXTENDPOLAR prompts the user to input a Polar structure and
    %   extends the angle of attack (ALPHAEXT), lift coefficient (CLEXT) and drag coefficient
    %   (CDEXT).
    %
    %   [ALPHAEXT, CLEXT, CDEXT] = EXTENDPOLAR(POLAR) uses the POLAR strcture given as input instead
    %   of prompting the user to select the file.
    %
    %   [ALPHAEXT, CLEXT, CDEXT] = EXTENDPOLAR(ALPHA, CL, CD) extends directly the values given in
    %   ALPHA, CL and CD over the whole range of angles of attack.
    %
    %   [...] = EXTENDPOLAR(..., 'method', METHOD) specifies the extension method.
    %   The available methods are:
    %     'Viterna'  - (default) Viterna & Corrigan. 1982. "Fixed Pitch Rotor
    %                  Performance of Large Horizontal Axis Wind Turbines".
    %
    %   [...] = EXTENDPOLAR(..., 'limit', LIMIT) specifies the point from where to extend.
    %   The available limits are:
    %     'last'  - (default) last point of the input polar
    %     'stall' - stall point, the points after stall are disregarded
    %
    % Inputs:
    %   Polar  : Polar structure (e.g. output from XF2MAT)
    %   alpha  : Vector or array with angles of attack [aoas x 1] or [aoas x nbPolar]
    %   cl     : Array with lift coefficients [aoas x nbPolar]
    %   cd     : Array with drag coefficients [aoas x nbPolar]
    %
    % Output:
    %   alphaExt : Extended range of angles of attack, IN RADIANS
    %   clExt    : Extended lift coefficient array
    %   cdExt    : Extended drag coefficient array
    %
    % Example:
    %   EXTENDPOLAR
    %   EXTENDPOLAR(Polar)
    %   EXTENDPOLAR(Polar, 'method', 'Viterna')
    %   EXTENDPOLAR(Polar, 'method', 'Viterna', 'limit', 'stall')
    %   EXTENDPOLAR(alpha, cl, cd)
    %   EXTENDPOLAR(alpha, cl, cd, 'method', 'Viterna', 'limit', 'stall')
    %
    % See also: XF2MAT, PLOTPOLARS.
    %
    % -----
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % Apache 2.0 License
    % https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox

    % ----------------------------------------------------------------------------------------------

    % Check number of inputs
    narginchk(0, 8); % Normally 7, but will disregard cm anyway, so 8 is ok

    % Import other functions from this package
    import af_tools.findstall
    import af_tools.utils.*

    % Defaults and constants
    OPTION_LIST = {'method', 'limit'};   % No options for this function

    % Extract and validate the inputs
    [alpha, c_l, c_d, ~, idxOpts] = parsepolarinputs(OPTION_LIST, varargin{:});
    if isempty(c_d)
        error('MATLAB:extendpolar:noCD', ...
              'To be extended, the polars must specify alpha, cl and cd.');
    end
    [method, limit] = parseoptioninputs(varargin{idxOpts:end});

    % Check if polars have enough data points to be extended
    if sign(min(alpha(:, 1))) == sign(max(alpha(:, 1)))
        error('MATLAB:extendpolar:noZero', ...
              'To be extended, a polar must at least contain the AOA = 0 deg.');
    end

    try
        findstall(alpha, c_l, c_d);
    catch
        error('MATLAB:extendpolar:noStall', ...
              'To be extended, a polar must at least contain the stall point.');
    end

    % -------------------------------------------
    % Extend polars

    if alpha(1, 1) == -pi || alpha(end, 1) == pi
        % If input already ranges from [-pi,pi], just send it back
        alpha_ext = alpha;
        c_l_ext = c_l;
        c_d_ext = c_d;
    else
        % Extend using the selected method
        switch method
            case 'Viterna'
                [alpha_ext, c_l_ext, c_d_ext] = viterna(limit, alpha, c_l, c_d);
        end
    end

end

% --------------------------------------------------------------------------------------------------
function [method, limit] = parseoptioninputs(varargin)
    % PARSEINPUTS Parses the input and checks their validity

    import af_tools.utils.*

    % Constants and defaults
    DEFAULT_METHOD = 'Viterna';
    DEFAULT_LIMIT = 'last';
    ALLOWED_METHODS = {'Viterna'};
    ALLOWED_LIMITS = {'stall', 'last'};

    % Option validator
    validMethod = @(x) any(validatestring(x, ALLOWED_METHODS, mfilename()));
    validLimit = @(x) any(validatestring(x, ALLOWED_LIMITS, mfilename()));

    % Parse options
    p = inputParser;
    addParameter(p, 'method', DEFAULT_METHOD, validMethod);
    addParameter(p, 'limit', DEFAULT_LIMIT, validLimit);
    parse(p, varargin{:});
    method = p.Results.method;
    limit = p.Results.limit;

end

% --------------------------------------------------------------------------------------------------
function [alpha_ext, c_l_ext, c_d_ext] = viterna(limit, alpha, c_l, c_d)
    % VITERNA Implementation of the original Viterna model
    % Source:
    %   Viterna & Corrigan. 1982. "Fixed Pitch Rotor Performance of Large Horizontal Axis Wind
    %   Turbines".

    % Note:
    %   The typical subscript 's' used in most definitions is here replaced by the generic 'l'
    %   (for limit) as the function can be used either with the stall point or the last available
    %   data.
    %
    % Process (the number will indicate the index of alpha, cl, cd):
    %   1. Extend Cl/Cd to ]limit, 90]
    %   2. Extend to ]90, 180-limit]
    %   3. Extend to ]180-limit,180] (lift must be corrected by linear interpolation)
    %   4. Extend to [-limit, -alpha(1)[
    %   5. Extend to [-90,-limit[
    %   6. Flip 3 and 4 to get the reminaing part
    %   EXTRA. Trim negative drag values to the minimum drag of the original polar

    % Import other functions from this package
    import af_tools.findstall
    import af_tools.utils.*

    % Constants
    DEFAULT_AR = 10; % AR as a rather small impact. 10 is a good compromise
    SCALING_CL = 0.7; % Suggested in AeroDyn's manual

    % Loop for every polar data (every column of cl)
    for i = 1:size(c_l, 2)

        % Trim the polars to nonNaN values only
        % (some Cl, Cd may be NaN if they come from xf2mat with trimAoas = false)
        id = find(~isnan(c_l(:, i)) == 1);
        trimmedAlpha = alpha(id, i);
        trimmedCl = c_l(id, i);
        trimmedCd = c_d(id, i);

        % Get limit point
        if strcmp(limit, 'stall')
            [alpha_l, cl_l, cd_l] = findstall(trimmedAlpha, trimmedCl, trimmedCd);
            % Trim polar values after stall point
            ids = find(trimmedAlpha > alpha_l);
            trimmedAlpha(ids) = [];
            trimmedCl(ids) = [];
            trimmedCd(ids) = [];
        elseif strcmp(limit, 'last')
            alpha_l = trimmedAlpha(end);
            cl_l = trimmedCl(end, :);
            cd_l = trimmedCd(end, :);
        end

        % Viterna's coefficients
        sas = sin(alpha_l);
        cas = cos(alpha_l);

        c_d_max = 1.11 + 0.018 * DEFAULT_AR;
        VitCoeffs.a1 = c_d_max / 2;
        VitCoeffs.b1 = c_d_max;
        VitCoeffs.a2 = (cl_l - c_d_max .* sas .* cas) .* sas ./ cas.^2;
        VitCoeffs.b2 = (cd_l - c_d_max .* sas.^2) ./ cas;

        % Minimum step between 2 AOA
        alpha_step = (min(diff(trimmedAlpha)));

        % 1. Extend Cl/Cd to ]limit, 90]
        alpha_1 = (alpha_l + alpha_step:alpha_step:pi / 2)';
        [cl1, cd1] = viternaeq(VitCoeffs, alpha_1, 1);

        % 2. Extend to ]90, 180-limit]
        alpha_2 = (pi / 2 + alpha_step:alpha_step:pi - alpha_l)';
        [c_l_2, c_d_2] = viternaeq(VitCoeffs, pi - alpha_2, -SCALING_CL);

        % 3. Extend to ]180-limit,180]
        alpha_3 = (pi - alpha_l + alpha_step:alpha_step:pi)';
        [~, c_d_3] = viternaeq(VitCoeffs, pi - alpha_3, 1);
        % Correct lift with linear variation
        c_l_3 = (alpha_3 - pi) / alpha_l * cl_l * SCALING_CL;

        % 4. Extend to [-limit, -alpha(1)[ (by linear interpolation)
        if trimmedAlpha(1) <= -alpha_l
            alpha_4 = [];
            c_l_4 = [];
            c_d_4 = [];
            alpha_lneg = trimmedAlpha(1);
        else
            alpha_4 = (-alpha_l:alpha_step:trimmedAlpha(1) - alpha_step)';
            c_l_4 = interp1([-alpha_l, trimmedAlpha(1)], [-cl_l * SCALING_CL, trimmedCl(1)], ...
                            alpha_4);
            c_d_4 = interp1([-alpha_l, trimmedAlpha(1)], [cd_l, trimmedCd(1)], alpha_4);
            alpha_lneg = -alpha_l;
        end

        % 5. Extend to [-90,-limit[
        alpha_5 = (-pi / 2:alpha_step:alpha_lneg - alpha_step)';
        [c_l_5, c_d_5] = viternaeq(VitCoeffs, alpha_5, SCALING_CL);

        % 6. Remaining part is simply a relfexion
        alpha_6 = (-pi:alpha_step:-pi / 2 - alpha_step)';
        c_l_6 = -flipud([c_l_2; c_l_3]);
        c_d_6 = flipud([c_d_2; c_d_3]);

        % Concatenate everything to get the full extended polars
        alpha_ext(:, i) = [alpha_6; alpha_5; alpha_4; trimmedAlpha; alpha_1; alpha_2; alpha_3];
        c_l_ext(:, i) = [c_l_6; c_l_5; c_l_4; trimmedCl; cl1; c_l_2; c_l_3];
        c_d_ext(:, i) = [c_d_6; c_d_5; c_d_4; trimmedCd; cd1; c_d_2; c_d_3];

        % Trim negatives cd to cdmin
        c_d_min = min(trimmedCd);
        c_d_ext(c_d_ext <= c_d_min, i) = c_d_min;
    end

end

% --------------------------------------------------------------------------------------------------
function [c_l, c_d] = viternaeq(VitCoeffs, alpha, clScaling)
    % VITERNAEQ Viterna's equation for the extension of the polars
    % Source:
    %   Viterna & Corrigan. 1982. "Fixed Pitch Rotor Performance of Large Horizontal Axis Wind
    %   Turbines".

    c_l = VitCoeffs.a1 * sin(2 * alpha) + VitCoeffs.a2 * cos(alpha).^2 ./ sin(alpha);
    c_l = c_l * clScaling;

    c_d = VitCoeffs.b1 * sin(alpha).^2 + VitCoeffs.b2  * cos(alpha);

end
