function [alpha_0, c_d_0] = findzerolift(varargin)
    % FINDZEROLIFT Finds the zero-lift angle and the associated cd.
    %   This function finds the zero-lift angle by interpolating around the last negative and first
    %   positive values for the lift coefficient.
    %
    %   FINDZEROLIFT accepts inputs in the form of a Polar structure (generated with XF2MAT) or
    %   values for angles of attack and their associated cl and cd.
    %
    %   If the input is specified as aoa, cl and cd, the three arrays must have the same number of
    %   rows (one for each angle of attack). The columns indicate values for different polars
    %   (different Mach, different Reynolds,...).
    %   For example, if the cl values are given for 2 different Reynols and there are 100 values for
    %   the aoa, cl and cd must be [100 x 2].
    %
    %   If extended polars are used as input (see EXTENDPOLAR), the function will only look for the
    %   zero-lift angle in the range of [-30, 30] deg.
    %
    % <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (README)</a>
    % <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues">Report an issue</a>
    % -----
    %
    % Usage:
    %   [ALPHA_0, CD_0] = FINDZEROLIFT prompts the user to input a Polar structure and
    %   determines the zero-lift angle of attack (ALPHA_0) and the drag coefficient at this
    %   angle (CD_0).
    %
    %   [ALPHA_0, CD_0] = FINDZEROLIFT(POLAR) uses the POLAR strcture given as input instead
    %   of prompting the user to select the file.
    %
    %   [ALPHA_0, CD_0] = FINDZEROLIFT(ALPHA, CL) uses directly the values given in in ALPHA
    %   and CL to determine the zero-lift angle. As no CD is provided, CD_0 will be an empty
    %   vector.
    %
    %   [ALPHA_0, CD_0] = FINDZEROLIFT(ALPHA, CL, CD) uses directly the values given in in
    %   ALPHA and CL to determine the zero-lift angle. As CD is provided, CD_0 will be returned
    %   as well.
    %
    % Inputs:
    %   Polar  : Polar structure (e.g. output from XF2MAT)
    %   alpha  : Vector or array with angles of attack [aoas x 1] or [aoas x nbPolar]
    %   cl     : Array with lift coefficients [aoas x nbPolar]
    %   cd     : Array with drag coefficients [aoas x nbPolar]
    %
    % Output:
    %   alpha_0 : Zero-lift angle of attack, IN RADIANS
    %   cd_0    : Drag at zero-lift angle of attack
    %
    % Example:
    %   FINDZEROLIFT
    %   FINDZEROLIFT(Polar)
    %   FINDZEROLIFT(alpha, cl)
    %   FINDZEROLIFT(alpha, cl, cd)
    %
    % See also: XF2MAT, FINDSTALL, FINDCLLINEARRANGE.
    %
    % -----
    % (c) Copyright 2022 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % Apache 2.0 License
    % https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox

    % ----------------------------------------------------------------------------------------------

    narginchk(0, 3);

    % Import other functions from this package
    import af_tools.utils.*

    % Defaults and constants
    ORIGINAL_RANGE = deg2rad([-30; 30]);  % Look for values in this range only
    OPTION_LIST = {};   % No options for this function

    % Extract and validate the inputs
    [alpha, c_l, c_d] = parsepolarinputs(OPTION_LIST, varargin{:});

    % Trim extended polars to original range
    % (avoid issues if extended polars are used as input)
    [alpha, c_l, c_d] = trimtorange(ORIGINAL_RANGE, alpha, c_l, c_d);

    % -------------------------------------------
    % Find zero lift for each polar

    [alpha_0, c_d_0] = zerofinder(alpha, c_l, c_d);

end

% --------------------------------------------------------------------------------------------------
function [alpha_0, c_d_0] = zerofinder(alpha, c_l, c_d)
    % FINDZEROLIFTVECT Finds the zero-lift point and returns the corresponding AOA and CD
    %   For vectors!

    % Process:
    %   1. Get the value of the last negative and first positive Cl
    %   2. Use linear interpolation between the two

    PLOT_POLAR_IF_ERROR = true; % Plot the polar if error so user can see why it fails

    % Initialization
    alpha_0 = zeros(1, size(c_l, 2));
    if isempty(c_d)
        c_d_0 = c_d;
    else
        c_d_0 = zeros(1, size(c_l, 2));
    end

    % Loop for every polar data (every column of cl)
    for i = 1:size(c_l, 2)

        idx = find(c_l(2:end, i) > 0, 1);  % Find first cl > 0
        idx = idx + 1; % Needs a +1 because we looked for cl(2:end)

        if isempty(idx)
            if PLOT_POLAR_IF_ERROR
                figure('Name', 'Debug: Polar with no zero lift');
                plot(rad2deg(alpha(:, i)), c_l(:, i));
                title('Polar with no clear zero lift angle');
                xlabel('AOA [deg]');
                ylabel('C_l [-]');
                grid on;
            end
            error('MATLAB:findzerolift:noPositiveCl', ['Impossible to find a '...
                                                       'positive Cl']);
        end

        if sign(c_l(idx, i)) == sign(c_l(idx - 1, i))
            if PLOT_POLAR_IF_ERROR
                figure('Name', 'Debug: Polar with no zero lift');
                plot(rad2deg(alpha(:, i)), c_l(:, i));
                title('Polar with no clear zero lift angle');
                xlabel('AOA [deg]');
                ylabel('C_l [-]');
                grid on;
            end
            error('MATLAB:findzerolift:noNegativeCl', ['Impossible to find a '...
                                                       'negative Cl']);
        end

        dummy(:, 1) = alpha(idx - 1:idx, i); % AOA of the two points around cl = 0
        dummy(:, 2) = c_l(idx - 1:idx, i); % Cl of the two points around cl = 0
        alpha_0(i) = interp1(dummy(:, 2), dummy(:, 1), 0);

        if ~isempty(c_d)
            c_d_0(i) = interp1(alpha(:, i), c_d(:, i), alpha_0(:, i));
        end

    end

end
